/*
 * Example program with a linked list of integers.
 * Also contains all of the pieces of a single-file C program,
 * including function prototypes.
 * CSE 303, 4/08, Hal Perkins
 */

#include <stdio.h>
#include <stdlib.h>

/* Data structure definitions */

struct Node {       /* Node in a linked list of integers: */
  int val;                  /*   Value in this node               */
  struct Node * next;       /*   Next node in list or null        */
};

/* function prototypes */
struct Node * new_node(int, struct Node *);
struct Node * make_list(int a[], int len);
void print_list(struct Node * lst);
void free_list(struct Node * lst);


/* Return a new Node with value v and next pointer next */
struct Node * new_node(int v, struct Node * next) {
  struct Node * ans;
  ans = (struct Node *)malloc(sizeof(struct Node *));
  ans->val = v;
  ans -> next = next;
  return ans;
}

/* Return a new linked list containing the value in a[0..len-1] in order */
struct Node * make_list(int a[], int len) {
  int k;
  struct Node * ans;
  ans = NULL;
  for (k = len-1; k >= 0; k--) {
    ans = new_node(a[k], ans);
  }
  return ans;
}

/* print the integers in the linked list lst */
void print_list(struct Node * lst) {
  struct Node * p;
  p = lst;
  while (p != NULL) {
    printf("%d ", p->val);
    p = p->next;
  }
  printf("\n");
}

/* Free the linked list lst */
void free_list(struct Node * lst) {
  struct Node * rest;   /* remainder of list not yet released */
  struct Node * p;
  rest = lst;
  while (rest != NULL) {
    p = rest;
    rest = rest->next;
    free(p);
  }
}

/* Create a list, print it, and free it */
int main() {
  struct Node * list;
  int nums[] = {17, 42, 303};
  list = make_list(nums, 3);
  print_list(list);
  free_list(list);
  return 0;
}
